'use strict';

// Fetches list of all service names if none are provided
var getServices = function () {
  var _ref = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee(ecs, cluster, services) {
    var list, serviceNames;
    return regeneratorRuntime.wrap(function _callee$(_context) {
      while (1) {
        switch (_context.prev = _context.next) {
          case 0:
            if (services.length) {
              _context.next = 6;
              break;
            }

            _context.next = 3;
            return ecs.listServices({ cluster: cluster, maxResults: 10 }).promise();

          case 3:
            list = _context.sent;
            serviceNames = list.serviceArns.map(extractNameFromARN);

            services.push.apply(services, _toConsumableArray(serviceNames));

          case 6:
            return _context.abrupt('return', services);

          case 7:
          case 'end':
            return _context.stop();
        }
      }
    }, _callee, this);
  }));

  return function getServices(_x, _x2, _x3) {
    return _ref.apply(this, arguments);
  };
}();

// Get the current task definition for each service


var getTaskDefs = function () {
  var _ref2 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee2(ecs, cluster, descriptions) {
    var getDefPromises;
    return regeneratorRuntime.wrap(function _callee2$(_context2) {
      while (1) {
        switch (_context2.prev = _context2.next) {
          case 0:
            getDefPromises = descriptions.services.map(function (service) {
              var taskDefinition = service.taskDefinition;

              logger.debug('current task def', { taskDefinition: taskDefinition });
              return ecs.describeTaskDefinition({ taskDefinition: taskDefinition }).promise();
            });
            return _context2.abrupt('return', Promise.all(getDefPromises));

          case 2:
          case 'end':
            return _context2.stop();
        }
      }
    }, _callee2, this);
  }));

  return function getTaskDefs(_x4, _x5, _x6) {
    return _ref2.apply(this, arguments);
  };
}();

// Returns a configured ECS client


// Registers an array of task definitions
var registerDefs = function () {
  var _ref3 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee3(ecs, newTaskDefs) {
    var registerDefPromises;
    return regeneratorRuntime.wrap(function _callee3$(_context3) {
      while (1) {
        switch (_context3.prev = _context3.next) {
          case 0:
            registerDefPromises = newTaskDefs.map(function (def) {
              return ecs.registerTaskDefinition(def).promise();
            });
            return _context3.abrupt('return', Promise.all(registerDefPromises));

          case 2:
          case 'end':
            return _context3.stop();
        }
      }
    }, _callee3, this);
  }));

  return function registerDefs(_x7, _x8) {
    return _ref3.apply(this, arguments);
  };
}();

// Updates services on a cluster to run new arns


var updateServices = function () {
  var _ref4 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee4(ecs, cluster, services, arns) {
    var updateServicePromises;
    return regeneratorRuntime.wrap(function _callee4$(_context4) {
      while (1) {
        switch (_context4.prev = _context4.next) {
          case 0:
            // Instruct services to use the new task definitions
            updateServicePromises = arns.map(function (taskDefArn, index) {
              var service = services[index];
              logger.info('updating ' + cluster + ':' + service + ' with ARN ' + taskDefArn);
              return ecs.updateService({
                cluster: cluster,
                service: service,
                taskDefinition: taskDefArn
              }).promise();
            });

            // Update services

            _context4.next = 3;
            return Promise.all(updateServicePromises);

          case 3:
            logger.info('waiting for services to stabilize...');

            // Wait for rollout
            _context4.next = 6;
            return ecs.waitFor('servicesStable', { cluster: cluster, services: services }).promise();

          case 6:
            logger.info('successfully updated the services');

          case 7:
          case 'end':
            return _context4.stop();
        }
      }
    }, _callee4, this);
  }));

  return function updateServices(_x9, _x10, _x11, _x12) {
    return _ref4.apply(this, arguments);
  };
}();

// Gets list of clusters (scoped to region)


var listClusters = function () {
  var _ref5 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee5(args) {
    var region, ecs, result, clusterNames;
    return regeneratorRuntime.wrap(function _callee5$(_context5) {
      while (1) {
        switch (_context5.prev = _context5.next) {
          case 0:
            region = args.region;
            ecs = getECS(region);
            _context5.next = 4;
            return ecs.listClusters().promise();

          case 4:
            result = _context5.sent;
            clusterNames = result.clusterArns.map(extractNameFromARN);
            // eslint-disable-next-line no-console

            console.log(prettyjson.render(clusterNames));

          case 7:
          case 'end':
            return _context5.stop();
        }
      }
    }, _callee5, this);
  }));

  return function listClusters(_x13) {
    return _ref5.apply(this, arguments);
  };
}();

// Gets list of services for ECS cluster (scoped to cluster)


var listServices = function () {
  var _ref6 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee6(args) {
    var region, cluster, ecs, result, serviceNames;
    return regeneratorRuntime.wrap(function _callee6$(_context6) {
      while (1) {
        switch (_context6.prev = _context6.next) {
          case 0:
            region = args.region, cluster = args.cluster;
            ecs = getECS(region);
            _context6.next = 4;
            return ecs.listServices({ cluster: cluster }).promise();

          case 4:
            result = _context6.sent;
            serviceNames = result.serviceArns.map(extractNameFromARN);
            // eslint-disable-next-line no-console

            console.log(prettyjson.render(serviceNames));

          case 7:
          case 'end':
            return _context6.stop();
        }
      }
    }, _callee6, this);
  }));

  return function listServices(_x14) {
    return _ref6.apply(this, arguments);
  };
}();

// Updates all ECS agents in a cluster
// TODO: listContainerInstances can take a filter param to scope this to a single instance


var updateAgents = function () {
  var _ref7 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee7(args) {
    var region, cluster, ecs, results, updateReqs;
    return regeneratorRuntime.wrap(function _callee7$(_context7) {
      while (1) {
        switch (_context7.prev = _context7.next) {
          case 0:
            region = args.region, cluster = args.cluster;
            ecs = getECS(region);
            _context7.next = 4;
            return ecs.listContainerInstances({ cluster: cluster }).promise();

          case 4:
            results = _context7.sent;

            logger.info('updating agents...');
            updateReqs = results.containerInstanceArns.map(function (arn) {
              return ecs.updateContainerAgent({ cluster: cluster, containerInstance: arn }).promise();
            });
            _context7.next = 9;
            return Promise.all(updateReqs);

          case 9:
            logger.info('updates requested');

          case 10:
          case 'end':
            return _context7.stop();
        }
      }
    }, _callee7, this);
  }));

  return function updateAgents(_x15) {
    return _ref7.apply(this, arguments);
  };
}();

// Deploys a service or set of services by running a new image


var deploy = function () {
  var _ref8 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee8(args) {
    var region, cluster, services, tag, ecs, serviceNames, descriptions, getDefPromises, taskDefs, newTaskDefs, registeredDefs, newArns;
    return regeneratorRuntime.wrap(function _callee8$(_context8) {
      while (1) {
        switch (_context8.prev = _context8.next) {
          case 0:
            region = args.region, cluster = args.cluster, services = args.services, tag = args.tag;


            logger.info('requested release', { cluster: cluster, services: services, tag: tag });
            ecs = getECS(region);
            _context8.next = 5;
            return getServices(ecs, cluster, services);

          case 5:
            serviceNames = _context8.sent;

            logger.info('targeting services', services);
            _context8.next = 9;
            return ecs.describeServices({ cluster: cluster, services: serviceNames }).promise();

          case 9:
            descriptions = _context8.sent;


            // Get the current task definition for each service
            getDefPromises = descriptions.services.map(function (service) {
              var taskDefinition = service.taskDefinition;

              logger.debug('current task def', { taskDefinition: taskDefinition });
              return ecs.describeTaskDefinition({ taskDefinition: taskDefinition }).promise();
            });
            _context8.next = 13;
            return Promise.all(getDefPromises);

          case 13:
            taskDefs = _context8.sent;


            // Make and register new definitions
            newTaskDefs = taskDefs.map(function (def) {
              return makeUpdatedDef(def, tag);
            });
            _context8.next = 17;
            return registerDefs(ecs, newTaskDefs);

          case 17:
            registeredDefs = _context8.sent;
            newArns = registeredDefs.map(function (def) {
              return def.taskDefinition.taskDefinitionArn;
            });
            return _context8.abrupt('return', updateServices(ecs, cluster, services, newArns));

          case 20:
          case 'end':
            return _context8.stop();
        }
      }
    }, _callee8, this);
  }));

  return function deploy(_x16) {
    return _ref8.apply(this, arguments);
  };
}();

// Rolls a services back to the previous task definitions


var rollback = function () {
  var _ref9 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee9(args) {
    var region, cluster, services, rev, ecs, serviceNames, descriptions, taskDefs, previousDefArns;
    return regeneratorRuntime.wrap(function _callee9$(_context9) {
      while (1) {
        switch (_context9.prev = _context9.next) {
          case 0:
            region = args.region, cluster = args.cluster, services = args.services, rev = args.rev;


            logger.info('requested rollback', { cluster: cluster, services: services, rev: rev });
            ecs = getECS(region);
            _context9.next = 5;
            return getServices(ecs, cluster, services);

          case 5:
            serviceNames = _context9.sent;

            logger.info('targeting services', services);
            _context9.next = 9;
            return ecs.describeServices({ cluster: cluster, services: serviceNames }).promise();

          case 9:
            descriptions = _context9.sent;
            _context9.next = 12;
            return getTaskDefs(ecs, cluster, descriptions);

          case 12:
            taskDefs = _context9.sent;
            previousDefArns = taskDefs.map(function (def) {
              var taskDef = def.taskDefinition;
              var family = taskDef.family;

              var relativeRev = rev || -1;

              if (relativeRev >= 0) {
                throw new Error('Relative revision must be a negative number');
              }

              var taskDefBase = taskDef.taskDefinitionArn.split('/' + family + ':')[0];
              var taskDefRev = taskDef.revision + relativeRev;
              var previousArn = taskDefBase + '/' + family + ':' + taskDefRev;
              return previousArn;
            });

            // TODO: ensure that all of the previous task definition revisions use
            // the same docker image

            return _context9.abrupt('return', updateServices(ecs, cluster, services, previousDefArns));

          case 15:
          case 'end':
            return _context9.stop();
        }
      }
    }, _callee9, this);
  }));

  return function rollback(_x17) {
    return _ref9.apply(this, arguments);
  };
}();

var configGet = function () {
  var _ref10 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee10(ecs, cluster, services, args, taskDefs) {
    var key, env, envVar;
    return regeneratorRuntime.wrap(function _callee10$(_context10) {
      while (1) {
        switch (_context10.prev = _context10.next) {
          case 0:
            key = args.key;

            logger.info('fetching ' + key, { cluster: cluster, services: services });
            env = taskDefs[0].taskDefinition.containerDefinitions[0].environment;
            envVar = env.find(function (x) {
              return x.name === key;
            });

            if (envVar) {
              // eslint-disable-next-line no-console
              console.log(envVar.value);
            }

          case 5:
          case 'end':
            return _context10.stop();
        }
      }
    }, _callee10, this);
  }));

  return function configGet(_x18, _x19, _x20, _x21, _x22) {
    return _ref10.apply(this, arguments);
  };
}();

var configSet = function () {
  var _ref11 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee11(ecs, cluster, services, args, taskDefs) {
    var key, val, newTaskDefs, registeredDefs, newArns;
    return regeneratorRuntime.wrap(function _callee11$(_context11) {
      while (1) {
        switch (_context11.prev = _context11.next) {
          case 0:
            key = args.key, val = args.val;

            logger.info('setting ' + key + '=' + val, { cluster: cluster, services: services });
            newTaskDefs = taskDefs.map(function (def) {
              var env = def.taskDefinition.containerDefinitions[0].environment;
              env.push({ name: key, value: val });
              return makeUpdatedDef(def, null, env);
            });
            _context11.next = 5;
            return registerDefs(ecs, newTaskDefs);

          case 5:
            registeredDefs = _context11.sent;
            newArns = registeredDefs.map(function (def) {
              return def.taskDefinition.taskDefinitionArn;
            });
            _context11.next = 9;
            return updateServices(ecs, cluster, services, newArns);

          case 9:
          case 'end':
            return _context11.stop();
        }
      }
    }, _callee11, this);
  }));

  return function configSet(_x23, _x24, _x25, _x26, _x27) {
    return _ref11.apply(this, arguments);
  };
}();

var configUnset = function () {
  var _ref12 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee12(ecs, cluster, services, args, taskDefs) {
    var key, newTaskDefs, registeredDefs, newArns;
    return regeneratorRuntime.wrap(function _callee12$(_context12) {
      while (1) {
        switch (_context12.prev = _context12.next) {
          case 0:
            key = args.key;

            logger.info('unsetting ' + key, { cluster: cluster, services: services });

            newTaskDefs = taskDefs.map(function (def) {
              var oldEnv = def.taskDefinition.containerDefinitions[0].environment;
              var newEnv = oldEnv.filter(function (envVar) {
                return envVar.name !== key;
              });
              return makeUpdatedDef(def, null, newEnv);
            });
            _context12.next = 5;
            return registerDefs(ecs, newTaskDefs);

          case 5:
            registeredDefs = _context12.sent;
            newArns = registeredDefs.map(function (def) {
              return def.taskDefinition.taskDefinitionArn;
            });
            _context12.next = 9;
            return updateServices(ecs, cluster, services, newArns);

          case 9:
          case 'end':
            return _context12.stop();
        }
      }
    }, _callee12, this);
  }));

  return function configUnset(_x28, _x29, _x30, _x31, _x32) {
    return _ref12.apply(this, arguments);
  };
}();

var configSubcommand = function () {
  var _ref13 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee13(ecs, cluster, services, args, taskDefs) {
    var subCommand;
    return regeneratorRuntime.wrap(function _callee13$(_context13) {
      while (1) {
        switch (_context13.prev = _context13.next) {
          case 0:
            subCommand = args._[1];
            _context13.t0 = subCommand;
            _context13.next = _context13.t0 === 'get' ? 4 : _context13.t0 === 'set' ? 6 : _context13.t0 === 'unset' ? 8 : 10;
            break;

          case 4:
            configGet(ecs, cluster, services, args, taskDefs);
            return _context13.abrupt('break', 11);

          case 6:
            configSet(ecs, cluster, services, args, taskDefs);
            return _context13.abrupt('break', 11);

          case 8:
            configUnset(ecs, cluster, services, args, taskDefs);
            return _context13.abrupt('break', 11);

          case 10:
            throw new Error('Invalid config subcommand ' + subCommand + '!');

          case 11:
          case 'end':
            return _context13.stop();
        }
      }
    }, _callee13, this);
  }));

  return function configSubcommand(_x33, _x34, _x35, _x36, _x37) {
    return _ref13.apply(this, arguments);
  };
}();

var configure = function () {
  var _ref14 = _asyncToGenerator( /*#__PURE__*/regeneratorRuntime.mark(function _callee14(args) {
    var region, cluster, services, ecs, numArgs, serviceNames, descriptions, taskDefs, defEnvs;
    return regeneratorRuntime.wrap(function _callee14$(_context14) {
      while (1) {
        switch (_context14.prev = _context14.next) {
          case 0:
            region = args.region, cluster = args.cluster, services = args.services;
            ecs = getECS(region);
            numArgs = args._.length;
            _context14.next = 5;
            return getServices(ecs, cluster, services);

          case 5:
            serviceNames = _context14.sent;
            _context14.next = 8;
            return ecs.describeServices({ cluster: cluster, services: serviceNames }).promise();

          case 8:
            descriptions = _context14.sent;
            _context14.next = 11;
            return getTaskDefs(ecs, cluster, descriptions);

          case 11:
            taskDefs = _context14.sent;


            if (numArgs === 1) {
              defEnvs = taskDefs.map(function (def) {
                var containerDef = def.taskDefinition.containerDefinitions[0];
                var familyName = def.taskDefinition.family;
                return {
                  name: familyName,
                  container: containerDef.name,
                  env: containerDef.environment.reduce(function (env, x) {
                    // eslint-disable-next-line no-param-reassign
                    env[x.name] = x.value;
                    return env;
                  }, {})
                };
              });

              defEnvs.forEach(function (defEnv) {
                if (defEnvs.length !== 1) {
                  // eslint-disable-next-line no-console
                  console.log('[TaskDef Family: ' + defEnv.name + ' :: Container: ' + defEnv.container + ']');
                }
                // eslint-disable-next-line no-console
                console.log(prettyjson.render(defEnv.env), '\n');
              });
            } else if (numArgs > 1) {
              configSubcommand(ecs, cluster, services, args, taskDefs);
            }

          case 13:
          case 'end':
            return _context14.stop();
        }
      }
    }, _callee14, this);
  }));

  return function configure(_x38) {
    return _ref14.apply(this, arguments);
  };
}();

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

var _ = require('lodash');
var AWS = require('aws-sdk');
var logger = require('winston');
var prettyjson = require('prettyjson');

var TAG_SEP = ':';
var PATH_SEP = '/';
var WRITABLE_TASK_DEF_PARAMS = ['containerDefinitions', 'volumes', 'family'];

// ECS resource ARN format:
// arn:aws:ecs:<region>:<account>:<resource_type>/<resource_name>
function extractNameFromARN(arn) {
  return arn.split(PATH_SEP)[1];
}function getECS(region) {
  AWS.config.update({ region: region });
  return new AWS.ECS({ apiVersion: '2014-11-13' });
}

// Updates task definition to specify image with provided tag
function makeUpdatedDef(def, imageTag, env) {
  // Keep only the important bits -- the api complains if we don't
  var newDef = _.pick(def.taskDefinition, WRITABLE_TASK_DEF_PARAMS);

  if (imageTag) {
    // Only support one container per def for now
    var containerDefs = newDef.containerDefinitions;
    var container = containerDefs[0];
    var namespace = container.image.split(TAG_SEP)[0];
    container.image = '' + namespace + TAG_SEP + imageTag;
  }

  if (env) {
    var _containerDefs = newDef.containerDefinitions;
    var _container = _containerDefs[0];
    _container.environment = env;
  }

  return newDef;
}

module.exports = {
  clusters: listClusters,
  services: listServices,
  deploy: deploy,
  rollback: rollback,
  configure: configure,
  updateAgents: updateAgents
};